/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <iostream>

#include <mysql.h>
#include <NdbApi.hpp>

#include "config.h"

using namespace std;

extern "C" int load_defaults(const char *conf_file, const char **groups,int *argc, char ***argv);

#define BAILOUT(obj, msg) do { fprintf(stderr, "%s: %s (%d) in %s:%d\n", msg, obj->getNdbError().message, obj->getNdbError().code, __FILE__, __LINE__); exit(3); } while (0);

void printBuffer(char *label, unsigned char *buff, int len)
{
  printf("%-16s: ", label);
  for (int i = 0; i < len; i++) {
	printf("%02X ", buff[i]);  
  }
  printf("\n");
}

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);

	// parse command line arguments
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	char *short_options = "c:";
	int opt, option_index;

	opterr = 0;
	do {
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
		switch (opt) {
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}



int main(int argc, char **argv)
{
  Ndb_cluster_connection *conn;
  char constr[1024];

  const char * tableName = "t1";

  conn = example_init(&argc, &argv);

  Ndb *ndb = new Ndb(conn, "test");
  if (!ndb) {
	fprintf(stderr, "Can't create NDB object\n");
	exit(3);
  }
  if (ndb->init()) {
	fprintf(stderr, "Can't create NDB object\n");
	exit(3);
  }

  NdbTransaction * trs = ndb->startTransaction();
  if (!trs) BAILOUT(ndb, "Can't start transaction");

  const NdbDictionary::Dictionary * dico = ndb->getDictionary();
  const NdbDictionary::Table * myTable = dico->getTable(tableName);

  NdbOperation * myOperation = trs->getNdbOperation(myTable);
  if (!myOperation) BAILOUT(trs, "Can't get operation");

  if (-1 == myOperation->readTuple(NdbOperation::LM_Read)) BAILOUT(myOperation, "readTuple() failed");
	
  // reading row with id=1
  Uint64 id = 1;  
  int crt = myOperation->equal("id", id);
  
  
  unsigned char buff32[16], buff33[16], buff63[16], buff64[16];

  memset(buff32, 0xff, sizeof(buff32));
  memset(buff33, 0xff, sizeof(buff33));
  memset(buff63, 0xff, sizeof(buff63));
  memset(buff64, 0xff, sizeof(buff64));
  
  NdbRecAttr * myRecAttr32 = myOperation->getValue(myTable->getColumn("bit32"), (char*)&buff32);
  NdbRecAttr * myRecAttr33 = myOperation->getValue(myTable->getColumn("bit33"), (char*)&buff33);
  NdbRecAttr * myRecAttr63 = myOperation->getValue(myTable->getColumn("bit63"), (char*)&buff63);
  NdbRecAttr * myRecAttr64 = myOperation->getValue(myTable->getColumn("bit64"), (char*)&buff64);
  
  printf("read buffers before execute:\n");
  printBuffer("BIT(32)", buff32, 16);
  printBuffer("BIT(33)", buff33, 16);
  printBuffer("BIT(63)", buff63, 16);
  printBuffer("BIT(64)", buff64, 16);
  printf("\n");

  if (-1 == trs->execute(NdbTransaction::NoCommit, NdbOperation::AO_IgnoreError)) BAILOUT(trs, "Execute failed");

  printf("read buffers before execute:\n");
  printBuffer("BIT(32)", buff32, 16);
  printBuffer("BIT(33)", buff33, 16);
  printBuffer("BIT(63)", buff63, 16);
  printBuffer("BIT(64)", buff64, 16);
  printf("\n");

  example_end(conn);

  return EXIT_SUCCESS;
}
